/************************************************************************/
/*																		*/
/*	System		: 78K0/IB2 Fluorecent Ballast Evaluation Board			*/
/*				  (EZ-BLST-003)											*/
/*																		*/
/*	Function	: Initialization										*/
/*																		*/
/*	CPU			: uPD78F0756											*/
/*																		*/
/*	Last Update	: 2009/08/10(MON)										*/
/*																		*/
/*	File name	: hdwini.c												*/
/*																		*/
/************************************************************************/
#pragma	SFR

/*========================================================================
	Include files
========================================================================*/
#include "defination.h"

/*========================================================================
	External function definition
========================================================================*/
extern void _wait10us(void);	/* wati for 10us under 2MHz CPU clock */
extern void _wait90us(void);	/* wati for 90us under 2MHz CPU clock */

/*************************************************************************
**	
**  function name:
**  	hdwinit
**
**	function:
**		Initialize Port, LVI, Clock, TMH1, TMX, ADC and CMP.
**		This function is called after reset.
**
**  Argument:
**  	void
**
**  Returned value:
**  	void
**
*************************************************************************/
void hdwinit(void)
{
	/*-------------------------------------------------------
		Memory size setting
	-------------------------------------------------------*/
	IMS = 0x04;					/* Internal ROM capacity: 16KB
								   Internal high-speed RAM capacity: 768bytes */
	
	/*-------------------------------------------------------
		Port setting
	-------------------------------------------------------*/
	PM0 = 0b11111111;
	/*      76543210
	        |||||||+--input, not used
	        ||||||+---input, not used
	        |||||+----input, not used
	        +++++-----input, default
	*/
	
	ADPC0 = 0b00000000;	/* All pins of P2 are setting to analog input mode */
	PM2 = 0b11111111;
	/*      76543210
	        |||||||+--input, DC output voltage detection for PFC control
	        ||||||+---input, not used
	        |||||+----input, not used
	        ||||+-----input, Zero current detection for PFC control
	        |||+------input, Lamp detection
	        ||+-------input, not used
	        |+--------input, AC supply voltage detection
	        +---------input, not used
	*/
	
	P3  = 0b00000000;
	/*      76543210
	        |||||||+--input,  not used
	        ||||||+---output, PFC output
	        |||||+----input,  not used
	        ||||+-----output, Low-side inverter output
	        |||+------output, High-side inverter output
	        +++-------input,  not used
	*/
	
	PM3 = 0b11100111;
	/*      76543210
	        |||||||+--input,  not used
	        ||||||+---input,  PFC output (change output mode when PFC operation start)
	        |||||+----input,  not used
	        ||||+-----output, Low-side inverter output
	        |||+------output, High-side inverter output
	        +++-------input,  not used
	*/
	
	PM6 = 0b11111111;
	/*      76543210
	        ||||||++--input, not used
	        ++++++----input, default
	*/
	
	ADPC1 = 0b00000000;	/* P70 is setting to analog input mode */
	PM7 = 0b11111111;
	/*      76543210
	        |||||||+--input, Volume detection for lamp dimming
	        +++++++---input, default
	*/
	
	/*-------------------------------------------------------
		Low voltage detector (LVI) setting
	-------------------------------------------------------*/
	if (!(RESF & 0b00000001))	/* If the reset source is not LVI, set LVI */
	{
		LVIMK = 1;				/* Mask the LVI interrupt */
		LVIS  = 0b00000000;		/* LVI occurs when 4.22V}0.1V */
		LVION = 1;				/* Enable LVI operation */
		_wait10us();			/* Wait for more than 10us. Clock works at 2MHz now */
		while (LVIF){};			/* Wait until VDD >= VLVI */
		LVIMD = 1;				/* RESET will occur when VDDVLVI */
	}
	
	/*-------------------------------------------------------
		Clock setting
		4MHz Internal CLOCK, x10=40MHz by PLL function
	-------------------------------------------------------*/
	while(!RSTS){}		/* Wait for the oscillation accuracy stabilization time of
						   internal high-speed oscillation clock */
	PLLON = 1;			/* Enable PLL operation */
	_wait90us();		/* Wait for more than 90us. Clock works at 2MHz now */
	SELPLL = 1;			/* Set the PLL mode */
	MCM = 0b00000000;
	/*      |||||||+--MCM0: 0, Internal high-speed oscillation clock(fIH)
            ||||||+---MCS:  0, Operates with internal high-speed oscillation clock
            |||||+----XSEL: 0, Internal high-speed oscillation clock(fIH)
            +++++-----must be 0
	*/
	PCC = 0b00000000;
	/*      |||||||+--PCC0: CPU clock (fCPU) selected to be fXP
            ||||||+---PCC1: CPU clock (fCPU) selected to be fXP
            |||||+----PCC2: CPU clock (fCPU) selected to be fXP
            +++++-----must be 0
	*/
	
	/*-------------------------------------------------------
		16-bit timer X0 setting for PFC control
	-------------------------------------------------------*/
	TX0CTL0 = 0b00000000;		/* TMX0 count clock selection */
	/*          |||||+++--TX0CKS2-0 : count clock selection
		        |||||	when SELPLL=0
		        |||||		 000 : fPRS
		        |||||		 001 : fPRS/2
		        |||||		 010 : fPRS/2^2
		        |||||		 011 : fPRS/2^3
		        |||||		 100 : fPRS/2^4
		        |||||		 101 : fPRS/2^5
		        |||||		 110 : fPRS/2^6
		        |||||		 111 : fPRS/2^7
		        |||||	when SELPLL=1
		        |||||		*000 : fTMX = 10*fXP = 40MHz
		        |||||		 001 : fPRS
		        |||||		other: prohibited
		        |||||
		        |++++----- must be 0
		        |
	            +---------TX0TMC : TMX0 count operation control
	            			*0 : count disable(stop)
	            			 1 : count enable
	*/
	
	TX0CTL1 = 0b00000000;		/* TMX0 output operation setting */
	/*          |||||+++--must be 0
		        ||||+-----TX0PWM: TMX0 PWM output operation setting
		        ||||		*0 : Single output
		        ||||		 1 : Dual output
		        ||||
		        ||++------TX0PWMCE,TX0PWMCINV: output gate function
		        ||			*00 : Does not use output gate function
		        ||			 01 : Does not use output gate function
		        ||			 10 : Output while TOH1 output is high level
		        ||			 11 : Output while TOH1 output is low level
		        ||
	            |+-------- must be 0
	            |
	            +---------TX0INTPST: Control of timer start operation
	            			*0 : Start while TX0TMC = 1
	            			 1 : Start with detection of INTP0 valid edge
	*/
	
/*	TX0CTL2 = 0b00000000;		/* No need to set this register in this application */
	/*		    |||||||+--TX0CCS: TX0CCR0 rigister control
			    |||||||		*0 : Operates as compare register
				|||||||		 1 : Operates as capture register
				|||||||
				||||||+---TX0ADEN: Control of generating A/D conversion synchronization trigger from TMX0
				||||||		*0 : Disables generating A/D conversion synchronization trigger
				||||||		 1 : Enables generating A/D conversion synchronization trigger
				||||||
				||++++---- must be 0
				||
				|+--------TX0TRGS: TMX0 capture trigger source selection
				|			*0 : INTCMP2
				|			 1 : INTP0
				|
				+--------- must be 0
	*/
	
	TX0CTL3 = 0b01100010;		/* Set the interlocking function with comparator 2 */
	/*	       	||||||++-- TX0CMP2RM1-0: Operation mode of interlocking function via comparator 2 output (interlocking with TMX0 timer)
				||||||			 00 : Disable this operation mode
				||||||			 01 : Resets timer when comparator 2 output is at active level
				||||||			*10 : Restarts timer when comparator 2 output is detected
				||||||			 11 : Resets timer output from detection of comparator 2 output to generation of next interrupt
				||||||
				||||++---- TX0INTP0RM1-0: Operation mode of interlocking function via INTP0
				||||			*00 : Disable this operation mode
				||||			 01 : Resets timer when INTP0 is at active level.
				||||			 10 : Restarts timer when INTP0 is detected
				||||			 11 : Resets timer output from detection of INTP0 to generation of next interrupt
		        ||||
		        |||+------ must be 0
		        |++------- TX0CMPLDSET1-0: Change of compare register when restarting upon comparator output
				|				 00 : Does not change compare register value when restarting
				|				 01 : Rewrites values of all compare registers at once when restarting upon comparator 0 output
				|				 10 : Rewrites values of all compare registers at once when restarting upon comparator 1 output
				|				*11 : Rewrites values of all compare registers at once when restarting upon comparator 2 output
	            |
		        +--------- must be 0
	*/
	
/*	TX0CTL4 = 0b00000000;		/* No need to set this register in this application */
	/*	       	||||||++-- TX0CMP0RM1-0: Operation mode of interlocking function via comparator 0 output (interlocking with TMX1 timer)
				||||||			*00 : Disable this operation mode
				||||||			 01 : Resets timer when comparator 0 output is at active level
				||||||			 10 : Restarts timer when comparator 0 output is detected
				||||||			 11 : Resets timer output from detection of comparator 0 output to generation of next interrupt
				||||||
				|||||+---- TX0CMP0RP: Selection of timer interlocking with comparator 0 output
				|||||			*0 : Comparator 0 output interlocks with TMX0 timer
				|||||			 1 : Comparator 0 output interlocks with TMX1 timer
				|||||
		       	|||++----- TX0CMP1RM1-0: Operation mode of interlocking function via comparator 1 output (interlocking with TMX1 timer)
				|||				*00 : Disable this operation mode
				|||				 01 : Resets timer when comparator 1 output is at active level
				|||				 10 : Restarts timer when comparator 1 output is detected
				|||				 11 : Resets timer output from detection of comparator 1 output to generation of next interrupt
		        |||
				||+------- TX0CMP1RP: Selection of timer interlocking with comparator 1 output
				||				*0 : Comparator 1 output interlocks with TMX0 timer
				||				 1 : Comparator 1 output interlocks with TMX1 timer
				||
		        ++-------- must be 0
	*/
	
	TX0IOC0 = 0b00000101;		/* TOX00 output setting */
	/*      	|||||||+--TX0TOL0: default value of TOX00
		        |||||||		 0 : set to low
		        |||||||		*1 : set to high
		        |||||||
		        ||||||+---TX0TOL1: default value of TOX01
		        ||||||		*0 : set to low
		        ||||||		 1 : set to high
		        ||||||
		        |||||+----TX0TOC0: TOX00 output control
		        |||||		 0 : disable PWM output
		        |||||		*1 : enable PWM output
		        |||||
		        ||||+-----TX0TOC1: TOX01 output control
		        ||||		*0 : disable PWM output
		        ||||		 1 : enable PWM output
		        ||||
  				++++------ must be 0
	*/
	
	/*-------------------------------------------------------
		16-bit timer X1 setting for inverter output
	-------------------------------------------------------*/
	TX1CTL0 = 0b00000000;		/* TMX1 count clock selection */
	/*         	|||||+++--TX1CKS2-0 : count clock selection
		        |||||	when SELPLL=0
		        |||||		 000 : fPRS
		        |||||		 001 : fPRS/2
		        |||||		 010 : fPRS/2^2
		        |||||		 011 : fPRS/2^3
		        |||||		 100 : fPRS/2^4
		        |||||		 101 : fPRS/2^5
		        |||||		 110 : fPRS/2^6
		        |||||		 111 : fPRS/2^7
		        |||||	when SELPLL=1
		        |||||		*000 : fTMX = 10*fXP = 40MHz
		        |||||		 001 : fPRS
		        |||||		other: prohibited
		        |||||
		        |++++----- must be 0
		        |
	            +---------TX1TMC : TMX1 count operation control
	            			*0 : count disable(stop)
	            			 1 : count enable
	*/
	
	TX1CTL1 = 0b00001000;		/* TMX1 output operation setting */
	/*       	||||||++--TX1MD1,0: Operation mode setting
		        ||||||		*00 : TMX1-only start mode
		        ||||||		 01 : TMX0 and TMX1 Synchronous start mode
		        ||||||		 10 : TMX0 and TMX1 Synchronous start/clear mode
		        ||||||		 11 : Prohibited
		        ||||||
		        |||||+---- must be 0
		        |||||
		        ||||+-----TX1PWM: TMX1 PWM output operation setting
		        ||||		 0 : Single output
		        ||||		*1 : Dual output
		        ||||
		        |||+------ must be 0
		        |||
		        ||+-------TX1PWMCE: Control of TOX1n output gate function
		        ||			*0 : Does not use output gate function
		        ||			 1 : Use output gate function
		        ++-------- must be 0
	*/
	
/*	TX1CTL2 = 0b00000000;		/* No need to set this register in this application */
	/*			|||||||+--TX1CCS: TX1CCR0 register operation
				|||||||		*0 : Operates as compare register
				|||||||		 1 : Operates as capture register
				|||||||
				||||||+---TX1ADEN:Control of generating A/D conversion synchronization trigger from TMX1
				||||||		*0 : Disables generating A/D conversion synchronization trigger
				||||||		 1 : Enables generating A/D conversion synchronization trigger
				||||||
				++++++---- must be 0
	*/
	
/*	TX1CTL4 = 0b00000000;		/* No need to set this register in this application */
	/*	       	||||||++-- TX1CMP0RM1-0: Operation mode of interlocking function via comparator 0 output (interlocking with TMX1 timer)
				||||||			*00 : Disable this operation mode
				||||||			 01 : Resets timer when comparator 0 output is at active level
				||||||			 10 : Restarts timer when comparator 0 output is detected
				||||||			 11 : Resets timer output from detection of comparator 0 output to generation of next interrupt
				||||||
				|||||+---- must be 0
				|||||
		       	|||++----- TX1CMP1RM1-0: Operation mode of interlocking function via comparator 1 output (interlocking with TMX1 timer)
				|||				*00 : Disable this operation mode
				|||				 01 : Resets timer when comparator 1 output is at active level
				|||				 10 : Restarts timer when comparator 1 output is detected
				|||				 11 : Resets timer output from detection of comparator 1 output to generation of next interrupt
		        |||
		        +++------- must be 0
	*/
	
	TX1IOC0 = 0b00001100;		/* TOX10 & TOX11 output setting */
	/*          |||||||+--TX1TOL0: default value of TOX10
		        |||||||		*0 : set to low
		        |||||||		 1 : set to high
		        |||||||
		        ||||||+---TX1TOL1: default value of TOX11
		        ||||||		*0 : set to low
		        ||||||		 1 : set to high
		        ||||||
		        |||||+----TX1TOC0: TOX10 output control
		        |||||		 0 : disable PWM output
		        |||||		*1 : enable PWM output
		        |||||
		        ||||+-----TX1TOC1: TOX11 output control
		        ||||		 0 : disable PWM output
		        ||||		*1 : enable PWM output
		        ||||
  				++++------ must be 0
  	*/
  	
	TX1CR0 = INV_DEAD_TIME - 1;		/* dead time setting (TX1CR0+1)*0.025[us] */
	
	/*-------------------------------------------------------
		8-bit timer H1 setting for system timer
	-------------------------------------------------------*/
	TMHMD1 = 0b01000000;
	/*         |||||||+--TOEN1: Timer output control
		       |||||||		*0 : Disables output
		       |||||||		 1 : Enables output
		       |||||||
		       ||||||+---TOLEV1: Timer output level control (in default mode)
		       ||||||		*0 : Low level
		       ||||||		 1 : High level
		       ||||||
		       ||||++----TMMD11,10: Timer operation mode
		       |||| 		*00 : Interval timer mode
		       |||| 		 01 : Carrier generator mode
		       ||||          10 : PWM output mode
		       ||||          11 : Setting prohibited
		       ||||
		       |+++------CKS12-10: Count clock selection
		       |    		 000 : fPRS
		       |    		 001 : fPRS/2^2
		       |    		 010 : fPRS/2^4
		       |    		 011 : fPRS/2^6
		       |    		*100 : fPRS/2^12
		       |    		 101 : fIL/2^6
		       |    		 110 : fIL/2^15
		       |    		 111 : fIL
               |
               +---------TMHE1: Timer operation enable
               				*0 : Stops timer count operation
               				 1 : Enables timer count operation
    */
	CMP01 = SYS_INTERVAL_TIME - 1;	/* Count the time as system timer. (CMP01+1)*0.2048[ms] */
	
	/*-------------------------------------------------------
		A/D converter setting
	-------------------------------------------------------*/
	ADM0 = 0b00000000;			/* Set the slowest conversion time (13.2us) */
	/*       |||||||+--ADCE: A/D voltage comparator operation control
			 |||||||	*0 : Stops A/D voltage comparator operation
			 |||||||	 1 : Enables A/D voltage comparator operation
		     ||||||+---LV0: refer to following table
		     |||||+----LV1: refer to following table
		     ||||+-----FR0: refer to following table
		     |||+------FR1: refer to following table
		     ||+-------FR2: refer to following table
		     |+--------must be 0
		     +---------ADCS:A/D conversion operation control
		     			*0 : Stops conversion operation
		     			 1 : Enables conversion operation
	
						 4.0 V <= AVREF <= 5.5 V
	 +-----------------------------------------------------------------+
	 |				ADM0			  |	Conversion Time Selection(us)  |
	 |--------------------------------+--------------------------------|
	 | FR2 | FR1 | FR0FR0 | LV1 | LV0 | 4MHz | 8MHz | 10MHz| 20MHz(PLL)|
	 |--------------------+-----------+--------------------------------|
	 |  0  |  0  |   0    |  0  |  0  | 66.0 | 33.0 | 26.4 | 13.2      |
 	 |--------------------|     |     |--------------------------------|
	 |  0  |  0  |   1    | 	|	  | 44.0 | 22.0 | 17.6 | 8.8       |
 	 |--------------------|     |     |--------------------------------|
	 |  0  |  1  |   0    |     |     | 33.0 | 16.5 | 13.2 | 6.6       |
 	 |--------------------|     |     |--------------------------------|
	 |  0  |  1  |   1    |     |     | 22.0 | 11.0 | 8.8  |           |
 	 |--------------------|     |     |--------------------|    X      |
	 |  1  |  0  |   0    |     |     | 16.5 | 8.25 | 6.6  | 	       |
 	 |--------------------|     |     |--------------------------------|
	 |  1  |  0  |   1    |     |     | 11.0 | 						   |
 	 |--------------------|     |     |------|			X			   |
	 |  1  |  1  |   0    |     |     | 8.25 | 						   |
 	 |--------------------|     |     |--------------------------------|
	 |  1  |  1  |   1    |     |     | 			X			       |
 	 |--------------------+-----------+--------------------------------|
	 |  1  |  0  |   1    |  1  |  1  | 11.0 | 5.5  | 4.4  |    X      |
 	 |--------------------|     |     |--------------------------------|
	 |  1  |  1  |   1    |     |     | 5.5  |           X   		   |
 	 |--------------------+-----------+--------------------------------|
	 |  1  |  0  |   0    |  1  |  0  | 16.5 | 8.25 | 6.6  |  *3.3     |
 	 |--------------------|     |     |--------------------------------|
	 |  1  |  1  |   0    |     |     | 8.25 | 4.125| 3.3  |    X	   |
 	 |--------------------+--------------------------------------------|
 	 |	      Other than above 		  | 			X				   |
	 +-----------------------------------------------------------------+
	*/
	
	/*-------------------------------------------------------
		Comparator setting
	-------------------------------------------------------*/
	C2RVM = 0b00011111;		/* Set the DA2 Internal Reference Voltage */
	/*        |||+++++--C2VRS4-0: DA2 Internal Reference Voltage Selection
	          |||        00000 : 0.05V (TYP.)
	          |||        00001 : 0.10V (TYP.)
	          |||        00010 : 0.15V (TYP.)
	          |||        00011 : 0.20V (TYP.)
	          |||        00100 : 0.25V (TYP.)
	          |||        00101 : 0.30V (TYP.)
	          |||        00110 : 0.35V (TYP.)
	          |||        00111 : 0.40V (TYP.)
	          |||        01000 : 0.45V (TYP.)
	          |||        01001 : 0.50V (TYP.)
	          |||        01010 : 0.55V (TYP.)
	          |||        01011 : 0.60V (TYP.)
	          |||        01100 : 0.65V (TYP.)
	          |||        01101 : 0.70V (TYP.)
	          |||        01110 : 0.75V (TYP.)
	          |||        01111 : 0.80V (TYP.)
	          |||        10000 : 0.85V (TYP.)
	          |||        10001 : 0.90V (TYP.)
	          |||        10010 : 0.95V (TYP.)
	          |||        10011 : 1.00V (TYP.)
	          |||        10100 : 1.05V (TYP.)
	          |||        10101 : 1.10V (TYP.)
	          |||        10110 : 1.15V (TYP.)
	          |||        10111 : 1.20V (TYP.)
	          |||        11000 : 1.25V (TYP.)
	          |||        11001 : 1.30V (TYP.)
	          |||        11010 : 1.35V (TYP.)
	          |||        11011 : 1.40V (TYP.)
	          |||        11100 : 1.45V (TYP.)
	          |||        11101 : 1.50V (TYP.)
	          |||        11110 : 1.55V (TYP.)
	          |||       *11111 : 1.60V (TYP.)
	          |||
	          +++-------must be 0
	*/
	CVRE = 1;				/* Enables internal reference voltage generation operation */
	C2CTL = 0b00010010;
	/*        |||||||+---C2INV: Output reversal setting
	          |||||||     *0 : Noninversion
	          |||||||      1 : Inversion
	          |||||||
	          ||||||+----C2OE: Enabling or disabling of comparator output
	          ||||||       0 : Disables output (output signal = fixed to low level)
	          ||||||      *1 : Enables output
	          ||||||
	          |||||+-----must be 0
	          |||||
	          |||++------C2MODSEL1-0: Reference voltage selection
	          |||          00 : Internal reference voltage: DA0
	          |||          01 : Internal reference voltage: DA1
	          |||         *10 : Internal reference voltage: DA2
	          |||          11 : Internal reference voltage: CMPCOM
	          |||
	          |++--------C2DFS1-0: Noise elimination width setting (fPRS = 20 MHz operation)
	          |           *00 : Noise filter unused
	          |            01 : 100ns
	          |            10 : 200ns
	          |            11 : 400ns
	          |
	          +----------CMP2EN: Comparator 2 operation control
	                      *0 : Stops operation
	                       1 : Enables operation
	*/
}
